#include <cmath>
#include "ccmap.h"
#include "common.h"
#include "logger.h"
#include "path.h"
#include "projectileanim.h"
#include "unit.h"
#include "unitandstructurepool.h"
#include "unitanimations.h"
#include "weaponspool.h"
#include "playerpool.h"

#define USE_OLD_MOVEANIM

#ifdef USE_OLD_MOVEANIM
UnitAnimEvent::UnitAnimEvent(Uint32 p, Unit* un) : ActionEvent(p)
{
    //logger->debug("UAE cons: this:%p un:%p\n",this,un);
    this->un = un;
    un->referTo();
    scheduled = NULL;
}

UnitAnimEvent::~UnitAnimEvent()
{
    //logger->debug("UAE dest: this:%p un:%p sch:%p\n",this,un,scheduled);
    if (scheduled != NULL) {
        p::aequeue->scheduleEvent(scheduled);
    }
    un->unrefer();
}

void UnitAnimEvent::setSchedule(UnitAnimEvent* e)
{
    //logger->debug("Scheduling an event. (this: %p, e: %p)\n",this,e);
    if (scheduled != NULL) {
        scheduled->setSchedule(NULL);
        scheduled->stop();
    }
    scheduled = e;
}

void UnitAnimEvent::stopScheduled()
{
    if (scheduled != NULL) {
        scheduled->stop();
    }
}

MoveAnimEvent::MoveAnimEvent(Uint32 p, Unit* un) : UnitAnimEvent(p,un)
{
    //logger->debug("MoveAnim cons (this %p un %p)\n",this,un);
    stopping = false;
    blocked = false;
    range = 0;
    this->dest = un->getTargetCell();
    this->un = un;
    path = NULL;
    newpos = 0xffff;
    istep = 0;
    moved_half = true;
    waiting = false;
    pathinvalid = true;
}

MoveAnimEvent::~MoveAnimEvent()
{
    if (un->moveanim == this)
        un->moveanim = NULL;
    //logger->debug("MoveAnim dest (this %p un %p dest %u)\n",this,un,dest);
    if( !moved_half && newpos != 0xffff) {
        p::uspool->abortMove(un,newpos);
    }
    delete path;
    if (un->walkanim != NULL) {
        un->walkanim->stop();
    }
    if (un->turnanim1 != NULL) {
        un->turnanim1->stop();

    }
    if (un->turnanim2 != NULL) {
        un->turnanim2->stop();
    }
}

void MoveAnimEvent::run()
{
    Sint8 uxoff, uyoff;
    Uint8 oldsubpos;
	Uint8 NewSubpos;

    waiting = false;
    if( !un->isAlive() ) {
        delete this;
        return;
    }

    if( path == NULL ) {
        p::uspool->setCostCalcOwnerAndType(un->owner, 0);
        path = new Path(un, un->getPos(), dest, range);
        if( !path->empty() ) {
            startMoveOne(false);
        } else {
            if (un->attackanim != NULL) {
                un->attackanim->stop();
            }
	    //printf ("Empty path --> stop anim??\n");
            delete this;
        }
        return;
    }

    if( blocked ) {
	//printf ("We are blocked\n");
        blocked = false;
        startMoveOne(true);
        return;
    }
    /* if distance left is smaller than xmod we're ready */

    un->xoffset += xmod;
    un->yoffset += ymod;

    if( !moved_half && (abs(un->xoffset) >= 12 || abs(un->yoffset) >= 12) ) {
        oldsubpos = un->subpos;
        NewSubpos = p::uspool->postMove(un, newpos);
		if (NewSubpos > 5){
			logger->error ("%s line %i: ****************** Move failed ****************", __FILE__, __LINE__);
			p::aequeue->scheduleEvent(this);
			return;
		}
		un->subpos = NewSubpos;
        un->cellpos = newpos;
        un->xoffset = -un->xoffset;
        un->yoffset = -un->yoffset;
        if( un->type->isInfantry() ) {
            un->infgrp->GetSubposOffsets(oldsubpos, un->subpos, &uxoff, &uyoff);
            un->xoffset += uxoff;
            un->yoffset += uyoff;
            xmod = 0;
            ymod = 0;
            if( un->xoffset < 0 )
                xmod = 1;
            else if( un->xoffset > 0 )
                xmod = -1;
            if( un->yoffset < 0 )
                ymod = 1;
            else if( un->yoffset > 0 )
                ymod = -1;
        }
        moved_half = true;
    }

    if( abs(un->xoffset) < abs(xmod) )
        xmod = 0;
    if( abs(un->yoffset) < abs(ymod) )
        ymod = 0;

    if( xmod == 0 && ymod == 0 ) {
        un->xoffset = 0;
        un->yoffset = 0;
        moveDone();
	//printf ("Move done, dest = %u, ourpos = %u\n", this->dest, un->getPos());
        return;
    }

    p::aequeue->scheduleEvent(this);
}

void MoveAnimEvent::startMoveOne(bool wasblocked)
{
Unit *BlockingUnit = NULL;
    Uint8 face;
#ifdef LOOPEND_TURN
    //TODO: transport boat is jerky (?)
    Uint8 loopend=((UnitType*)un->type)->getAnimInfo().loopend;
#endif
    newpos = p::uspool->preMove(un, path->top(), &xmod, &ymod, &BlockingUnit);

#if 0
	if ( un->getOwner() == p::ppool->getLPlayerNum() ){
		Uint16 x, y;
		p::ccmap->translateFromPos(newpos, &x, &y);
		printf ("%s line %i: Newpos = %i [%i:%i]\n", __FILE__, __LINE__, newpos, x, y);
	}
#endif
    if( newpos == 0xffff ) {
        delete path;
        path = NULL;
        p::uspool->setCostCalcOwnerAndType(un->owner, 0);
        path = new Path(un, un->getPos(), dest, range);
        pathinvalid = false;
        if( path->empty() ) {
            logger->note ("%s line %i: path is empty\n", __FILE__, __LINE__);
            xmod = 0;
            ymod = 0;
            p::aequeue->scheduleEvent(this);
            return;
        }
        newpos = p::uspool->preMove(un, path->top(), &xmod, &ymod, &BlockingUnit);
        if( newpos == 0xffff ) {
            if (wasblocked) {
                xmod = 0;
                ymod = 0;
                if (un->attackanim != NULL) {
                    un->attackanim->stop();
                }
                this->stop();
                p::aequeue->scheduleEvent(this);
                return;
            } else {
                /* TODO: tell the blocking unit to move here */

//                logger->note ("%s line %i: We are blocked\n", __FILE__, __LINE__);

                blocked = true;

                if (un->walkanim != NULL) {
                    un->walkanim->stop();
                }

                p::aequeue->scheduleEvent(this);
                return;
            }
        }
    }

#ifdef LOOPEND_TURN 
    face = ((Sint8)((loopend+1)*(8-path->top())/8))&loopend;
#else
    face = (32-(path->top() << 2))&0x1f;

//	if (un->getOwner() == p::ppool->getLPlayerNum())
//		printf ("%s line %i: TurnAnimEvent face(dir) = %i, path->top = %i, (path->top() << 2) = %i\n", __FILE__, __LINE__, face, path->top(), (path->top() << 2));
#endif
    path->pop();

	//printf ("%s line %i: Face = %u\n", __FILE__, __LINE__, face);

    moved_half = false;

    if (((UnitType *)un->getType())->isInfantry()) {
        if (un->walkanim != NULL) {
            un->walkanim->changedir(face);
        } else {
            un->walkanim = new WalkAnimEvent(((UnitType *)un->getType())->getSpeed(), un, face, 0);
            p::aequeue->scheduleEvent(un->walkanim);
        }
        p::aequeue->scheduleEvent(this);

    } else {
#ifdef LOOPEND_TURN 
        Uint8 curface = (un->getImageNum(0)&loopend);
        Uint8 delta = (abs(curface-face))&loopend;
//        if( curface != face ) {
//            if( (delta <= (Sint8)((loopend+1)/8)) || (delta >= (Sint8)(loopend*7/8))) {
		// Don't try to turn if we are already turning
        if( curface != face && un->turnanim1 == NULL) {
            if( ((delta <= (Sint8)((loopend+1)/8)) || (delta >= (Sint8)(loopend*7/8))) || un->getType()->getType() == UN_PLANE) {
#else
        Uint8 curface = (un->getImageNum(0)&0x1f);
        Uint8 delta = (abs(curface-face))&0x1f;
        if( (un->getImageNum(0)&0x1f) != face ) {
            if( (delta < 5) || (delta > 27) ) {
#endif
				//printf ("%s line %i: Turn 1\n", __FILE__, __LINE__);
                un->turn(face,0);
                p::aequeue->scheduleEvent(this);
            } else {
				//printf ("%s line %i: Turn 2\n", __FILE__, __LINE__);
                waiting = true;
                un->turn(face,0);
                un->turnanim1->setSchedule(this);
            }
            if (un->getType()->getNumLayers() > 1) {
                un->turn(face,1);
            }
        } else {
            p::aequeue->scheduleEvent(this);
        }
    }
}

void MoveAnimEvent::moveDone()
{
//	printf ("Move done\n");
    un->xoffset = 0;
    un->yoffset = 0;

    if (pathinvalid) {
        delete path;
        p::uspool->setCostCalcOwnerAndType(un->owner, 0);
        path = new Path(un, un->getPos(), dest, range);
        pathinvalid = false;
    }
    if( !path->empty() && !stopping ) {
        startMoveOne(false);
    } else {
        if( dest != un->getPos() && !stopping ) {
            delete path;
            p::uspool->setCostCalcOwnerAndType(un->owner, 0);
            path = new Path(un, un->getPos(), dest, range);
            pathinvalid = false;
        }
        if( path->empty() || stopping ) {
            delete this;
        } else {
            startMoveOne(false);
        }
    }
}

void MoveAnimEvent::stop()
{
    stopping = true;
    //stopScheduled();
}

void MoveAnimEvent::update()
{
    //logger->debug("Move updating\n");
    dest = un->getTargetCell();
    pathinvalid = true;
    stopping = false;
    range = 0;
}


#else
UnitAnimEvent::UnitAnimEvent(Uint32 p, Unit* un) : ActionEvent(p)
{
    //logger->debug("UAE cons: this:%p un:%p\n",this,un);
    this->un = un;
    un->referTo();
    scheduled = NULL;
}

UnitAnimEvent::~UnitAnimEvent()
{
    //logger->debug("UAE dest: this:%p un:%p sch:%p\n",this,un,scheduled);
    if (scheduled != NULL) {
        p::aequeue->scheduleEvent(scheduled);
    }
    un->unrefer();
}

void UnitAnimEvent::setSchedule(UnitAnimEvent* e)
{
    //logger->debug("Scheduling an event. (this: %p, e: %p)\n",this,e);
    if (scheduled != NULL) {
        scheduled->setSchedule(NULL);
        scheduled->stop();
    }
    scheduled = e;
}

void UnitAnimEvent::stopScheduled()
{
    if (scheduled != NULL) {
        scheduled->stop();
    }
}

MoveAnimEvent::MoveAnimEvent(Uint32 p, Unit* un) : UnitAnimEvent(p,un)
{
    //logger->debug("MoveAnim cons (this %p un %p)\n",this,un);
	DefaultDelay = p;
    stopping = false;
    blocked = false;
    range = 0;
    this->dest = un->getTargetCell();
    this->un = un;
    path = NULL;
    newpos = 0xffff;
    istep = 0;
    moved_half = true;
    waiting = false;
    pathinvalid = true;
	BlockedCounter = 0;
}

MoveAnimEvent::~MoveAnimEvent()
{
	if (un->moveanim == this)
        un->moveanim = NULL;
    //logger->debug("MoveAnim dest (this %p un %p dest %u)\n",this,un,dest);
    if( !moved_half && newpos != 0xffff) {
        p::uspool->abortMove(un,newpos);
    }
    delete path;
    if (un->walkanim != NULL) {
        un->walkanim->stop();
    }
    if (un->turnanim1 != NULL) {
        un->turnanim1->stop();

    }
    if (un->turnanim2 != NULL) {
        un->turnanim2->stop();
    }
}

void MoveAnimEvent::run()
{
    Sint8 uxoff, uyoff;
    Uint8 oldsubpos;
	Uint8 NewSubpos;

    waiting = false;
    if( !un->isAlive() ) {
        delete this;
        return;
    }

    if( path == NULL ) {
        p::uspool->setCostCalcOwnerAndType(un->owner, 0);
        path = new Path(un, un->getPos(), dest, range);
        if( !path->empty() ) {
            startMoveOne(false);
        } else {
            if (un->attackanim != NULL) {
                un->attackanim->stop();
            }
	    //printf ("Empty path --> stop anim??\n");
            delete this;
        }
        return;
    }

    if( blocked ) {
	//printf ("We are blocked\n");
        blocked = false;
        startMoveOne(true);
        return;
    }
#if 0
else if (BlockedCounter > 0 && !stopping){
        startMoveOne(false);
		return;
	}
#endif

    /* if distance left is smaller than xmod we're ready */
    un->xoffset += xmod;
    un->yoffset += ymod;

    if( !moved_half && (abs(un->xoffset) >= 12 || abs(un->yoffset) >= 12) ) {
        oldsubpos = un->subpos;
        NewSubpos = p::uspool->postMove(un, newpos);
		if (NewSubpos > 5){
			logger->error ("%s line %i: ****************** Move failed ****************", __FILE__, __LINE__);
			p::aequeue->scheduleEvent(this);
			return;
		}
		un->subpos = NewSubpos;
        un->cellpos = newpos;
        un->xoffset = -un->xoffset;
        un->yoffset = -un->yoffset;
        if( un->type->isInfantry() ) {
            un->infgrp->GetSubposOffsets(oldsubpos, un->subpos, &uxoff, &uyoff);
            un->xoffset += uxoff;
            un->yoffset += uyoff;
            xmod = 0;
            ymod = 0;
            if( un->xoffset < 0 )
                xmod = 1;
            else if( un->xoffset > 0 )
                xmod = -1;
            if( un->yoffset < 0 )
                ymod = 1;
            else if( un->yoffset > 0 )
                ymod = -1;
        }
        moved_half = true;
    }

    if( abs(un->xoffset) < abs(xmod) )
        xmod = 0;
    if( abs(un->yoffset) < abs(ymod) )
        ymod = 0;

    if( xmod == 0 && ymod == 0 ) {
        un->xoffset = 0;
        un->yoffset = 0;
        moveDone();
	//printf ("Move done, dest = %u, ourpos = %u\n", this->dest, un->getPos());
        return;
    }

    p::aequeue->scheduleEvent(this);
}

void MoveAnimEvent::startMoveOne(bool wasblocked)
{
Unit *BlockingUnit = NULL;

    Uint8 face;
#ifdef LOOPEND_TURN
    //TODO: transport boat is jerky (?)
    Uint8 loopend=((UnitType*)un->type)->getAnimInfo().loopend;
#endif
    newpos = p::uspool->preMove(un, path->top(), &xmod, &ymod, &BlockingUnit);

#if 0
	if ( un->getOwner() == p::ppool->getLPlayerNum() ){
		Uint16 x, y;
		p::ccmap->translateFromPos(newpos, &x, &y);
		printf ("%s line %i: Newpos = %i [%i:%i]\n", __FILE__, __LINE__, newpos, x, y);
	}
#endif
    if( newpos == 0xffff ) {
        delete path;
        path = NULL;
        p::uspool->setCostCalcOwnerAndType(un->owner, 0);
        path = new Path(un, un->getPos(), dest, range);
        pathinvalid = false;
        if( path->empty() ) {
            logger->note ("%s line %i: path is empty\n", __FILE__, __LINE__);
            xmod = 0;
            ymod = 0;
            p::aequeue->scheduleEvent(this);
            return;
        }
        newpos = p::uspool->preMove(un, path->top(), &xmod, &ymod, &BlockingUnit);
        if( newpos == 0xffff ) {
            if (wasblocked) {
                xmod = 0;
                ymod = 0;
                if (un->attackanim != NULL) {
                    un->attackanim->stop();
                }
                this->stop();
                p::aequeue->scheduleEvent(this);
                return;
            } else {
                /* TODO: tell the blocking unit to move here */

//                logger->note ("%s line %i: We are blocked\n", __FILE__, __LINE__);

#if 0
				if (BlockingUnit != NULL){
					printf ("Blocked by unit %s\n", BlockingUnit->getType()->getTName());
				}

				xmod = ymod = 0;
				BlockedCounter++;

				if (BlockingUnit == NULL){
                	blocked = true;
					BlockedCounter = 0;
				}else{
					if (!BlockingUnit->IsMoving () || BlockedCounter > 30)
						blocked = true;
						BlockedCounter = 0;
				}

				setDelay(14);
#endif
				xmod = ymod = 0;
                if (un->walkanim != NULL) {
                    un->walkanim->stop();
                }

                p::aequeue->scheduleEvent(this);
                return;
            }
        }
    }

	setDelay( DefaultDelay );
	BlockedCounter = 0;

#ifdef LOOPEND_TURN 
    face = ((Sint8)((loopend+1)*(8-path->top())/8))&loopend;
#else
    face = (32-(path->top() << 2))&0x1f;

//	if (un->getOwner() == p::ppool->getLPlayerNum())
//		printf ("%s line %i: TurnAnimEvent face(dir) = %i, path->top = %i, (path->top() << 2) = %i\n", __FILE__, __LINE__, face, path->top(), (path->top() << 2));
#endif
    path->pop();

	//printf ("%s line %i: Face = %u\n", __FILE__, __LINE__, face);

    moved_half = false;

    if (((UnitType *)un->getType())->isInfantry()) {
        if (un->walkanim != NULL) {
            un->walkanim->changedir(face);
        } else {
            un->walkanim = new WalkAnimEvent(((UnitType *)un->getType())->getSpeed(), un, face, 0);
            p::aequeue->scheduleEvent(un->walkanim);
        }
        p::aequeue->scheduleEvent(this);

    } else {
#ifdef LOOPEND_TURN 
        Uint8 curface = (un->getImageNum(0)&loopend);
        Uint8 delta = (abs(curface-face))&loopend;
//        if( curface != face ) {
//            if( (delta <= (Sint8)((loopend+1)/8)) || (delta >= (Sint8)(loopend*7/8))) {
		// Don't try to turn if we are already turning
        if( curface != face  && un->turnanim1 == NULL) {
            if( ((delta <= (Sint8)((loopend+1)/8)) || (delta >= (Sint8)(loopend*7/8))) || un->getType()->getType() == UN_PLANE) {
#else
        Uint8 curface = (un->getImageNum(0)&0x1f);
        Uint8 delta = (abs(curface-face))&0x1f;
        if( (un->getImageNum(0)&0x1f) != face ) {
            if( (delta < 5) || (delta > 27) ) {
#endif
				//printf ("%s line %i: Turn 1\n", __FILE__, __LINE__);
                un->turn(face,0);
                p::aequeue->scheduleEvent(this);
            } else {
				//printf ("%s line %i: Turn 2\n", __FILE__, __LINE__);
                waiting = true;
                un->turn(face,0);
                un->turnanim1->setSchedule(this);
            }
            if (un->getType()->getNumLayers() > 1) {
                un->turn(face,1);
            }
        } else {
            p::aequeue->scheduleEvent(this);
        }
    }
}

void MoveAnimEvent::moveDone()
{
//	printf ("Move done\n");
    un->xoffset = 0;
    un->yoffset = 0;

    if (pathinvalid) {
        delete path;
        p::uspool->setCostCalcOwnerAndType(un->owner, 0);
        path = new Path(un, un->getPos(), dest, range);
        pathinvalid = false;
    }
    if( !path->empty() && !stopping ) {
        startMoveOne(false);
    } else {
        if( dest != un->getPos() && !stopping ) {
            delete path;
            p::uspool->setCostCalcOwnerAndType(un->owner, 0);
            path = new Path(un, un->getPos(), dest, range);
            pathinvalid = false;
        }
        if( path->empty() || stopping ) {
            delete this;
        } else {
            startMoveOne(false);
        }
    }
}

void MoveAnimEvent::stop()
{
    stopping = true;
    //stopScheduled();
}

void MoveAnimEvent::update()
{
    //logger->debug("Move updating\n");
    dest = un->getTargetCell();
    pathinvalid = true;
    stopping = false;
    range = 0;
}
#endif

WalkAnimEvent::WalkAnimEvent(Uint32 p, Unit *un, Uint8 dir, Uint8 layer) : UnitAnimEvent(p,un)
{
    //fprintf(stderr,"debug: WalkAnim constructor\n");
    this->un = un;
    this->dir = dir;
    this->layer = layer;
    stopping = false;
    istep = 0;
    calcbaseimage();
}

WalkAnimEvent::~WalkAnimEvent()
{
#ifdef LOOPEND_TURN
    un->setImageNum((((UnitType*)un->type)->getAnimInfo().loopend+1)*dir/8,layer);
#else
	un->setImageNum( dir>>2, layer );
#endif
    if (un->walkanim == this)
        un->walkanim = NULL;
}

void WalkAnimEvent::run()
{
    Uint8 layerface;
    if (!stopping) {
        layerface = baseimage + istep;
        // XXX: Assumes 6 frames to loop over
        istep = (istep + 1)%6;
		//printf ("%s line %i: Baseimg = %i, istep = %i\n", __FILE__, __LINE__, baseimage, istep);
		un->setImageNum(layerface,layer);
        p::aequeue->scheduleEvent(this);
    } else {
        delete this;
        return;
    }
}

void WalkAnimEvent::calcbaseimage()
{
    // XXX: this is really nasty, will be taken care of after the rewrite
    baseimage = 16 + 3*(dir/2);
}

TurnAnimEvent::TurnAnimEvent(Uint32 p, Unit *un, Uint8 dir, Uint8 layer) : UnitAnimEvent(p,un)
{
#ifdef LOOPEND_TURN   
    Uint8 loopend=((UnitType*)un->type)->getAnimInfo().loopend;
#endif
    //logger->debug("Turn cons (t%p u%p d%i l%i)\n",this,un,dir,layer);
    Uint8 layerface;
    this->un = un;
    this->dir = dir;
    this->layer = layer;
    stopping = false;
#ifdef LOOPEND_TURN 
    layerface = un->getImageNum(layer)&loopend;
    if( ((layerface-dir)&loopend) < ((dir-layerface)&loopend) ) {
#else
    layerface = un->getImageNum(layer)&0x1f;
    if( ((layerface-dir)&0x1f) < ((dir-layerface)&0x1f) ) {
#endif
        turnmod = -(((UnitType *)un->getType())->getTurnMod());
    } else {
        turnmod = (((UnitType *)un->getType())->getTurnMod());
    }
}

TurnAnimEvent::~TurnAnimEvent()
{
    //logger->debug("TurnAnim dest\n");
    if (layer == 0) {
        if (un->turnanim1 == this)
            un->turnanim1 = NULL;
    } else {
        if (un->turnanim2 == this)
            un->turnanim2 = NULL;
    }
}

void TurnAnimEvent::run()
{
#ifdef LOOPEND_TURN   
    Uint8 loopend=((UnitType*)un->type)->getAnimInfo().loopend;
#endif
    Uint8 layerface;

    //logger->debug("TurnAnim run (s%i)\n",stopping);
    if (stopping) {
        delete this;
        return;
    }

#ifdef LOOPEND_TURN   
    layerface = un->getImageNum(layer)&loopend;
    if( abs((layerface-dir)&loopend) > abs(turnmod) ) {
        layerface += turnmod;
        layerface &= loopend;
#else
    layerface = un->getImageNum(layer)&0x1f;
    if( abs((layerface-dir)&0x1f) > abs(turnmod) ) {
        layerface += turnmod;
        layerface &= 0x1f;
#endif
    } else
        layerface = dir;
//		if (un->getOwner() == p::ppool->getLPlayerNum())
//			printf ("%s line %i: TurnAnimEvent layerface %i, dir = %i, turnmod = %i\n", __FILE__, __LINE__, layerface, dir, turnmod);
		un->setImageNum(layerface,layer);

    if( layerface == dir || !un->isAlive()) {
        delete this;
        return;
    }
    p::aequeue->scheduleEvent(this);
}

UAttackAnimEvent::UAttackAnimEvent(Uint32 p, Unit *un) : UnitAnimEvent(p,un)
{
    //logger->debug("UAttack cons\n");
    this->un = un;
    this->target = un->getTarget();
    stopping = false;
    waiting = 0;
    target->referTo();
	Weapon *Weap;

	UsePrimaryWeapon = true;

	// Determine the weapon to use
	if (!target->getType()->isStructure()){
		switch (((Unit*)target)->getType()->getType()){
			case UN_INFANTRY:
			case UN_VEHICLE:
				Weap = un->getType()->getWeapon();
				if (Weap != NULL){
					if (!Weap->getProjectile()->AntiGround()){
						UsePrimaryWeapon = false;
					}
				}
				break;
			case UN_BOAT:
				Weap = un->getType()->getWeapon();
				if (Weap != NULL){
					if (!Weap->getProjectile()->AntiGround()){
						UsePrimaryWeapon = false;
					}
				}
				break;
			case UN_PLANE:
			case UN_HELICOPTER:
				Weap = un->getType()->getWeapon();
				if (Weap != NULL){
					if (!Weap->getProjectile()->AntiAir()){
						UsePrimaryWeapon = false;
					}
				}
				break;
			default:
				logger->error ("%s line %i: ERROR unknown unit type %i\n", __FILE__, __LINE__, ((Unit*)target)->getType()->getType());
				break;
		}
	}else{
		Weap = un->getType()->getWeapon();
		if (Weap != NULL){
			if (!Weap->getProjectile()->AntiGround()){
				UsePrimaryWeapon = false;
			}
		}
	}
	if (UsePrimaryWeapon == false){
//		printf ("%s line %i: Using secundary weapon\n", __FILE__, __LINE__);
		if (un->getType()->getWeapon(UsePrimaryWeapon) == NULL){
			logger->error ("Primary weapon not oke, secundary weapon not available\n");
			UsePrimaryWeapon = true;
			if (un->getType()->getWeapon(UsePrimaryWeapon) == NULL){
				stop();
			}
		}
	}
}

UAttackAnimEvent::~UAttackAnimEvent()
{
    //logger->debug("UAttack dest\n");
    target->unrefer();
    if (un->attackanim == this)
        un->attackanim = NULL;
}

void UAttackAnimEvent::update()
{
    //logger->debug("UAtk updating\n");
    target->unrefer();
    target = un->getTarget();
    target->referTo();
    stopping = false;
}

void UAttackAnimEvent::stop()
{
    if (un == NULL) {
        logger->error("UAttackAnimEvent::stop: un is NULL!?\n");
        abort();
    }
    stopping = true;
}

void UAttackAnimEvent::run()
{
    Uint32 distance;
    Sint32 xtiles, ytiles;
    Uint16 atkpos;
    float alpha;
    Uint8 facing;
#ifdef LOOPEND_TURN
    Uint8 loopend2=((UnitType*)un->type)->getAnimInfo().loopend2;
#endif

    //logger->debug("attack run t%p u%p\n",this,un);
    waiting = 0;
    if( !un->isAlive() || stopping ) {
        delete this;
        return;
    }

    if( !target->isAlive() || stopping) {
        if ( !target->isAlive() ) {
            un->doRandTalk(TB_postkill);
        }
        delete this;
        return;
    }

	// Check if we have a healing weapon (medic) and if we have healed someone completely
	if (un->getType()->getWeapon(UsePrimaryWeapon)->getDamage() < 0){
		// Oke, this is a healing weapon
		if ( target->getHealth() ==  target->getType()->getMaxHealth()){
			delete this;
			return;
		}
	}

    atkpos = un->getTargetCell();

    xtiles = un->cellpos % p::ccmap->getWidth() - atkpos % p::ccmap->getWidth();
    ytiles = un->cellpos / p::ccmap->getWidth() - atkpos / p::ccmap->getWidth();
    distance = abs(xtiles)>abs(ytiles)?abs(xtiles):abs(ytiles);

    if( distance > un->type->getWeapon(UsePrimaryWeapon)->getRange() /* weapons range */ ) {
        setDelay(0);
        waiting = 3;
        un->move(atkpos,false);
        un->moveanim->setRange(un->type->getWeapon(UsePrimaryWeapon)->getRange());
        un->moveanim->setSchedule(this);
        return;
    }
    //Make sure we're facing the right way
    if( xtiles == 0 ) {
        if( ytiles < 0 ) {
            alpha = -M_PI_2;
        } else {
            alpha = M_PI_2;
        }
    } else {
        alpha = atan((float)ytiles/(float)xtiles);
        if( xtiles < 0 ) {
            alpha = M_PI+alpha;
        }
    }
#ifdef LOOPEND_TURN
    facing = ((Sint8)((loopend2+1)*(1-alpha/2/M_PI)+8))&loopend2;
    if (un->type->isInfantry()) {
        if (facing != (un->getImageNum(0)&loopend2)) {
            un->setImageNum((Sint8)((loopend2+1)*facing/8),0);
        }
    } else if (un->type->getNumLayers() > 1 ) {
        if (abs((int)(facing - (un->getImageNum(1)&loopend2))) > un->type->getROT()) {
#else
    facing = (40-(Sint8)(alpha*16/M_PI))&0x1f;
    if (un->type->isInfantry()) {
        if (facing != (un->getImageNum(0)&0x1f)) {
			un->setImageNum(facing>>2,0);
        }
    } else if (un->type->getNumLayers() > 1 ) {
        if (abs((int)(facing - (un->getImageNum(1)&0x1f))) > un->type->getROT()) {
#endif
            setDelay(0);
            waiting = 2;
            un->turn(facing,1);
            un->turnanim2->setSchedule(this);
            return;
        }
    } else {
#ifdef LOOPEND_TURN
        if (abs((int)(facing - un->getImageNum(0)&loopend2)) > un->type->getROT()) {
#else
        if (abs((int)(facing - un->getImageNum(0)&0x1f)) > un->type->getROT()) {
#endif
            setDelay(0);
            waiting = 1;
            un->turn(facing,0);
            un->turnanim1->setSchedule(this);
            return;
        }
    }

	// If we have a healing weapon we don't want to heal a enemy
	if (un->getType()->getWeapon(UsePrimaryWeapon)->getDamage() < 0){
		if (un->getOwner() != target->getOwner()){
			delete this;
			return;
		}
	}

    // We can shoot
    un->type->getWeapon(UsePrimaryWeapon)->fire(un, target->getBPos(un->getPos()), target->getSubpos());
    // set delay to reloadtime
    setDelay(un->type->getWeapon(UsePrimaryWeapon)->getReloadTime());
    waiting = 4;
    p::aequeue->scheduleEvent(this);
}

//#define DEBUG_HARVEST_ANIM
UHarvestEvent::UHarvestEvent(Uint32 p, Unit *un) : UnitAnimEvent(p,un)
{
#ifdef DEBUG_HARVEST_ANIM
	logger->debug("Harvest init\n");
#endif
	ForceEmpty			= false;
	this->un			= un;
	stopping			= false;
	index				= 0;
	delay				= 0;
	facing				= 0;
	MoveTargePos		= un->FindTiberium ();
	manual_pauze		= false;
	NumbResources		= un->GetNumResources ();
	new_orgimage		= false;
	OrgImage			= un->getImageNum(0)&0xff;
	RetryMoveCounter	= 0;
	ReturnStep			= HARVEST_RETURN_STEP_1;

	// Get the resources whe harvested previously from the unit
	for (int i = 0; i < NumbResources; i++)
		un->GetResourceType (i, &ResourceTypes[i]);
}

UHarvestEvent::~UHarvestEvent()
{
	logger->debug("UHarvest destructor\n");

	if (this->un->harvestanim == this)
		this->un->harvestanim = NULL;
}

void UHarvestEvent::update()
{
    stopping = false;
}


void UHarvestEvent::setHarvestingPos(Uint32 pos)
{
	MoveTargePos = pos;

	if ( p::uspool->getStructureAt(pos) != NULL){
 		if (p::uspool->getStructureAt(pos)->isRefinery ()){
			manual_pauze = false;
			if (NumbResources > 0){
				ReturnStep	= HARVEST_RETURN_STEP_1;
				ForceEmpty	= true;
			}
			return;
		}
	}

	// Pauze harvesting if the player sends us to a place where there is no resource
	if (p::ccmap->getResourceFrame(pos) == 0){
		manual_pauze = true;
		return;
	}

	manual_pauze = false;
}


void UHarvestEvent::stop()
{
    if (un == NULL) {
#ifdef DEBUG_HARVEST_ANIM
        printf("UHarvestEvent::stop: un is NULL!?\n");
#endif
        abort();
    }
    stopping = true;
}

int UHarvestEvent::GetBaseRefineryPos (void){
Uint16 x, y, xdiv;
int temppos;

	if (un->GetBaseRefinery() == NULL)
		return 0;

	xdiv = 1;

	p::ccmap->translateFromPos(un->GetBaseRefinery()->getPos(), &x, &y);
	y += 2;
	temppos = p::ccmap->translateToPos(x+xdiv, y);

//	while ( !p::ccmap->isBuildableAt(un->getOwner(), temppos) && y < p::ccmap->getHeight() && un->getPos() != temppos ){
	while ( p::ccmap->getCost(temppos, un) > 4 && y < p::ccmap->getHeight() && un->getPos() != temppos ){
		if ( xdiv < 10 && x + xdiv < p::ccmap->getWidth())
			xdiv++;
		else{
			y += 1;
			xdiv = 0;
		}
		//printf ("xdiv is %i\n", xdiv);
		temppos = p::ccmap->translateToPos(x+xdiv, y);
	}

//	printf ("UNITANIMATIONS x = %i, y = %i, temppos = %i\n", x, y, temppos);

	return temppos;
}

void UHarvestEvent::run()
{
#ifdef DEBUG_HARVEST_ANIM
	if ( un->getOwner() == p::ppool->getLPlayerNum() )
		printf ("%s line %i: Run harvest animation\n", __FILE__, __LINE__);
#endif

	if( !un->isAlive() || stopping ) {
#ifdef DEBUG_HARVEST_ANIM
		if ( un->getOwner() == p::ppool->getLPlayerNum() )
			printf ("%s line %i: Stopping harvest animation\n", __FILE__, __LINE__);
#endif
		delete this;
		return;
	}

	if (RetryMoveCounter > 5){
		stop();
		setDelay(14);
		p::aequeue->scheduleEvent(this);
		return;
	}


	if (un->GetBaseRefinery() != NULL){
		if (!un->GetBaseRefinery()->isAlive())
			un->SetBaseRefinery(NULL);
	}

	if (manual_pauze){
#ifdef DEBUG_HARVEST_ANIM
		if ( un->getOwner() == p::ppool->getLPlayerNum() )
			printf ("%s line %i: Manual pauze harvest animation\n", __FILE__, __LINE__);
#endif
		un->setImageNum(OrgImage, 0);
		new_orgimage = true;
       	if (un->getPos() == MoveTargePos){
			stop();
			setDelay(14);
			p::aequeue->scheduleEvent(this);
			return;
		}
		un->move(MoveTargePos, false);
		if (RetryMoveCounter == 0){
			un->moveanim->setSchedule(this);
		}else{
			setDelay(14);
			p::aequeue->scheduleEvent(this);
		}
		RetryMoveCounter++;
		return;
	}

	if (new_orgimage){
		OrgImage = un->getImageNum(0)&0xff;
		OrgImage -= OrgImage%4;
		if (OrgImage > 32)
			OrgImage = 8;
		//facing = (Uint32) OrgImage;
		new_orgimage = false;
		//printf ("New org image\n");
	}

	if (NumbResources < 5 && !ForceEmpty/*&& !un->EmptyHarvester*/){
#ifdef DEBUG_HARVEST_ANIM
		if ( un->getOwner() == p::ppool->getLPlayerNum() )
			printf ("%s line %i: Harvest\n", __FILE__, __LINE__);
#endif
		if (un->getPos() != MoveTargePos){
			// If there is something occupying our position move to a diffirent pos...
//			if (p::uspool->getUnitOrStructureAt(MoveTargePos) != NULL && un->getPos() != MoveTargePos || RetryMoveCounter > 2){
			if (p::uspool->cellOccupied(MoveTargePos) && un->getPos() != MoveTargePos || RetryMoveCounter > 2){
				MoveTargePos = un->FindTiberium ();
			}
			un->setImageNum(OrgImage, 0);
			un->move(MoveTargePos, false);
			new_orgimage = true;
			if (RetryMoveCounter == 0){
				un->moveanim->setSchedule(this);
			}else{
				setDelay(14);
				p::aequeue->scheduleEvent(this);
			}
			RetryMoveCounter++;
			return;
		}
		RetryMoveCounter = 0;

        if (p::ccmap->getResourceFrame(un->getPos()) == 0){
            MoveTargePos = un->FindTiberium ();
        }

        int NumbLayers = un->type->getNumLayers();

        facing = (Uint32)(((double)OrgImage / (double)32) * (double)8);

        if (index < 8){
            un->setImageNum(32+(facing*8) + index, NumbLayers - 1);
            index++;
        }else{
            //un->setImageNum(OrgImage, NumbLayers - 1);

            // Start new annimation
            index = 0;

            Uint8 type, amount;

            // Remove the mined tiberium from the map
            if (p::ccmap->getResource(un->getPos(), &type, &amount)) {
                p::ccmap->decreaseResource(un->getPos(), 1);
                ResourceTypes[NumbResources] = type;
                un->AddResource (ResourceTypes[NumbResources]);
                //if (un->getOwner() == p::ppool->getLPlayerNum())
                    //printf ("LPlayer, add resource type %i to harvester\n", type);
                NumbResources++;
                ReturnStep = HARVEST_RETURN_STEP_1;
            }
        }
    }else if (un->GetBaseRefinery() != NULL){
#ifdef DEBUG_HARVEST_ANIM
		if ( un->getOwner() == p::ppool->getLPlayerNum() )
			printf ("%s line %i: Empty, step = %i \n", __FILE__, __LINE__, ReturnStep);
#endif
		// We are full --> move back to our base and dump our content there
		//int MoveToPos = this->GetBaseRefineryPos ();
		if (ReturnStep == HARVEST_RETURN_STEP_1){ 
			if (this->GetBaseRefineryPos () != un->getPos()){
				un->setImageNum(OrgImage, 0);
		        un->move(this->GetBaseRefineryPos (), false);
				new_orgimage = true;
				if (RetryMoveCounter == 0){
					un->moveanim->setSchedule(this);
				}else{
					setDelay(14);
					p::aequeue->scheduleEvent(this);
				}
				RetryMoveCounter++;
				return;
			}else{
				RetryMoveCounter = 0;
				ReturnStep = HARVEST_RETURN_STEP_2;
			}

		}else if (ReturnStep == HARVEST_RETURN_STEP_2){ //un->getImageNum(0)&0xff != 8){
                	un->turn(8,0);
                	un->turnanim1->setSchedule(this);
			new_orgimage = true;
			ReturnStep = HARVEST_RETURN_STEP_3;
			return;
		}else if (ReturnStep == HARVEST_RETURN_STEP_3){
			// We are back at the refinery, start dump animation
			int NumbLayers = un->type->getNumLayers();

			if (index < 8){
				un->setImageNum(32+(8*8) + index, NumbLayers - 1);
				index++;
			}else{

				un->setImageNum(OrgImage, NumbLayers - 1);

//			if (index > 8){
				// Start new annimation
				index = 0;

				int Value = 0;
				for (int i = 0; i < NumbResources; i++)
					Value += (10 - ResourceTypes[i])*50;

				NumbResources = 0;

                un->EmptyResources ();

				ReturnStep = HARVEST_RETURN_STEP_1;

				ForceEmpty = false;

				p::ppool->getPlayer(un->getOwner())->changeMoney(Value);

				if (un->getOwner() != p::ppool->getLPlayerNum())
					MoveTargePos		= un->FindTiberium ();
			}
		}else{
			// Hmm, we should never get here --> make returnstep 1
			ReturnStep = HARVEST_RETURN_STEP_1;
		}
	}
	// Reschedule this..
	setDelay(10);
	p::aequeue->scheduleEvent(this);

}


URepairEvent::URepairEvent(Uint32 p, Unit *un) : UnitAnimEvent(p,un)
{
int		un_cost;
Sint16	health;

	this->un	= un;
	stopping	= false;
	index		= 0;
	delay		= 0;

	fix_str_num = un->GetFixStr();
	fix_str_pos = un->GetFixPos ();

	health = un->getHealth();
	un_cost = un->getType()->getCost();
	if (health > 0)
		dmg_cost = (Uint16)(((double)un_cost/(double)un->getType()->getMaxHealth()) * ((double)un->getType()->getMaxHealth() - (double)health));
	else
		dmg_cost = (Uint16)un_cost;

	moveCounter = 0;
}

URepairEvent::~URepairEvent()
{
    if (un->repairanim == this)
        un->repairanim = NULL;
}

void URepairEvent::update()
{
    stopping = false;
}


void URepairEvent::stop()
{
    if (un == NULL) {
        printf("Repair unit::stop: un is NULL!?\n");
        abort();
    }
    stopping = true;
}


void URepairEvent::run()
{
Structure *FixStr;

	if( !un->isAlive() || stopping ) {
		delete this;
		return;
	}

	FixStr = p::uspool->getStructureAt(fix_str_pos);

	if ( FixStr == NULL ){
		delete this;
		return;
	}

	// Check the structure is actually fix
	if (strcmp ((char*)FixStr->getType()->getTName(), "FIX") != 0 ){
		delete this;
		return;
	}

	if (un->getPos() != fix_str_pos){
		if (moveCounter > 4){
			// Can't reach target, give up
			printf ("%s line %i: Can't reach target --> give up\n", __FILE__, __LINE__);
			delete this;
			return;
		}
//		if ((un->getType()->getTName(), "HARV") == 0 ){
//			un->Harvest (fix_str_pos, NULL);
//		}

		un->move(fix_str_pos, true);
		un->moveanim->setSchedule(this);
		moveCounter++;
		return;
	}

	if (!FixStr->RepairUnint (un)){
		delete this;
		return;	
	}

	// Oke, the rest has to be done by the fix structure --> stop
	stop();

	// Reschedule this..
	setDelay(14);
	p::aequeue->scheduleEvent(this);

}

