#include "SDL.h"
#include "common.h"
#include "config.h"
#include "cpsimage.h"
#include "graphicsengine.h"
#include "logger.h"
#include "pause_menu.h"
#include "cursor.h"
#include "sidebar.h"
#include "shpimage.h"
#include "vfs.h"
#include "vqa.h"
#include "wsa.h"
#include "soundengine.h"
#include "sdllayer.h"

PauseMenuClass::PauseMenuClass():MenuFont("type.fnt"), VersionFont ("grad6fnt.fnt")
{
	// Setup the font color
	Font_color.r = 255;
	Font_color.g = 255;
	Font_color.b = 255;

	display = pc::gfxeng->get_SDL_ScreenSurface ();

	// Load the cursor
	cursorimg = new Dune2Image("mouse.shp", -1);
	if (cursorimg == NULL)
		logger->error("Couldn't load cursor image\n");

	my_cursor = cursorimg->getImage(0);

	//
	// Setup the first popup window
	//
	PauseWindow.SetupWindow(130, 90, 400, 250);

	//
	// Draw the buttons on the new window..
	//
	OptionsButton.SetDrawingWindow (&PauseWindow);
	OptionsButton.CreateSurface("game options", 105, 110, 180, 22 );
	ExitButton.SetDrawingWindow (&PauseWindow);
	ExitButton.CreateSurface("abort Mission", 105, 145, 180, 22 );
	ContinueButton.SetDrawingWindow (&PauseWindow);
	ContinueButton.CreateSurface("Resume Mission", 30, 200, 180, 22 );

	MenuFont.drawText("Options", PauseWindow.GetWindowSurface (), Font_color, 160, 30) ;
	VersionFont.drawText(VERSION.c_str(), PauseWindow.GetWindowSurface (), Font_color, 290, 210) ;

	for (int x = 162; x < 228; x++)
		set_pixel ( PauseWindow.GetWindowSurface (), 0xffffff, x, 45 );
}

PauseMenuClass::~PauseMenuClass()
{

	if (my_cursor != NULL)
		SDL_FreeSurface(my_cursor);
}

void PauseMenuClass::DrawMousePointer (void)
{
SDL_Rect	dest;
int 		mx, my;

	SDL_GetMouseState(&mx, &my);

	//
	// Actually draw the mouse
	//
	dest.x = mx;
	dest.y = my;
	if (dest.x < 0)
		dest.x = 0;
	if (dest.y < 0)
		dest.y = 0;
	dest.w = my_cursor->w;
	dest.h = my_cursor->h;

	SDL_BlitSurface(my_cursor, NULL, display, &dest);
}


void PauseMenuClass::HandleInput (void)
{
SDL_Event	event;

	if (SDL_WaitEvent(&event)){

		// Pass events on to diffirent menu items
		if (ExitButton.handleMouseEvent (event))
			DrawMousePointer ();
		if (ContinueButton.handleMouseEvent (event))
			DrawMousePointer ();

		// Handle the diffirent mouse and keyboard effents
		switch (event.type){

			case SDL_MOUSEBUTTONDOWN:
				break;

			case SDL_MOUSEBUTTONUP:
				if (event.button.button == SDL_BUTTON_LEFT){
					if (ExitButton.MouseOver()){
						// exit to main menu
						pc::Config.quit_mission = true;
						pc::Config.pause = false;

						// Stop all the music
						pc::sfxeng->StopMusic();
						pc::sfxeng->StopLoopedSound(-1);
						//pc::sfxeng->ResumeLoopedSound(-1);

						for (int i = 0; i < p::ppool->getNumPlayers(); i++)
							p::ppool->getPlayer(i)->resumeBuilding();
					}
					if (ContinueButton.MouseOver()){
						// Continue the game
						pc::Config.pause = false;
						pc::sfxeng->ResumeLoopedSound(-1);
						for (int i = 0; i < p::ppool->getNumPlayers(); i++)
							p::ppool->getPlayer(i)->resumeBuilding();
					}
				}
				break;
			case SDL_KEYDOWN:
				if (event.key.keysym.sym == SDLK_ESCAPE){
					pc::Config.pause = false;
				}
				break;
		}
	}else{
		printf ("Error");
	}
}

int PauseMenuClass::HandleMenu(void)
{
int				mx,
				my,
				old_mx = 0,
				old_my = 100;
std::string		TmpString;
TCheckBox		CheckBox;

	pc::sfxeng->PauseLoopedSound(-1);

	for (int i = 0; i < p::ppool->getNumPlayers(); i++)
		p::ppool->getPlayer(i)->pauseBuilding();

	SDL_GetMouseState(&mx, &my);

	this->HandleInput ();

	// Keep rendering the scene as background
	pc::gfxeng->renderScene(false);

	// Draw the buttons on the window
	OptionsButton.drawbutton ();
	ExitButton.drawbutton ();
	ContinueButton.drawbutton ();

	// Draw the window (whith the buttons on it) to the screen
	PauseWindow.DrawWindow();

	this->DrawMousePointer ();

	SDL_Flip(display);

	old_mx = mx;
	old_my = my;

    return 0;
}
